#!env python
import sys
import datetime
import os.path
from argparse import ArgumentParser
from math import ceil
from githubstar.fileexporter import StarsExporter
from githubstar.version import VERSION
from github import Github
from urllib3 import Retry


def printProgress(p, base=0):
    if p < 0:
        p = 0
    elif p > 100:
        p = 100
    print("\rProgress: {}%".format(p + base), end="")


def getFilePathName(args):
    if args.destname:
        filename = args.destname
    else:
        dateBackup = datetime.datetime.now().strftime("%Y%m%d%H%M%S")
        prefixname = "github-stars-" + args.username
        match args.format:
            case "bookmark":
                filename = prefixname + "-" + "bookmark" + "-" + dateBackup + ".html"
            case "md":
                filename = prefixname + "-" + dateBackup + ".md"
            case "json":
                filename = prefixname + "-" + dateBackup + ".json"
            case _:
                filename = prefixname + "-" + dateBackup + ".html"
    if args.destpath:
        if os.path.exists(args.destpath) and os.path.isdir(args.destpath):
            filename = os.path.normpath(os.path.join(args.destpath + "\\", filename))
        else:
            print("Invalid destpath:" + args.destpath, file=sys.stderr)
            sys.exit()
    return filename


def starred_repos(user):
    starred = user.get_starred()
    total_pages = ceil(starred.totalCount / 30)
    for page_num in range(0, total_pages):
        for repo in starred.get_page(page_num):
            yield repo
        printProgress(int(95 * (page_num + 1) / total_pages), 2)


def retryConfig(backoff_factor=1.0, total=8):
    """urllib3 will sleep for:
        {backoff factor} * (2 ** ({number of total retries} - 1))
    Recalculates and Overrides Retry.DEFAULT_BACKOFF_MAX"""
    Retry.DEFAULT_BACKOFF_MAX = backoff_factor * 2 ** (total - 1)
    return Retry(total=total, backoff_factor=backoff_factor)


def parse_args():
    parser = ArgumentParser(prog='githubstar', description="Export a GitHub user's starred list to local file.")
    parser.add_argument('--version', action='version', version=VERSION)
    parser.add_argument("--username", dest='username', required=True, help="[required]username to export from")
    parser.add_argument("--token", dest='token', required=False,
                        help="token from https://github.com/settings/tokens, to avoid rate-limiting, can also store in environment as 'GITHUB_TOKEN'.")
    parser.add_argument("--format", dest='format', default="html", required=False,
                        choices=['html', 'bookmark', 'md', 'json'], help="output format, default: html")
    parser.add_argument("--groupby", dest='groupby', default="none", required=False,
                        choices=['none', 'language', 'topic'], help="default: none")
    parser.add_argument("--orderby", dest='orderby', default="timestarred", required=False,
                        choices=['timestarred', 'timeupdated', 'alphabet', 'starscount', 'forkscount', 'language'],
                        help="default: timestarred")
    parser.add_argument("--orderdirection", dest="orderdirection", default="desc", required=False,
                        choices=["desc", "asc"], help="default: desc")
    parser.add_argument("--ordernum", dest="showOrderNum", default="true", required=False,
                        choices=["true", "false"],
                        help="show order number before repository name or not, default: true")
    parser.add_argument("--excludeprivate", dest="excludeprivate", default="false", required=False,
                        choices=["true", "false"],
                        help="exclude private repositories, default: false")
    parser.add_argument("--destpath", dest='destpath', required=False, help="path to store the exported file")
    parser.add_argument("--destname", dest='destname', required=False, help="filename of the exported file")
    return parser.parse_args()


def main():
    args = parse_args()
    if not args.username:
        print("Please set `--username` to a valid GitHub user name.", file=sys.stderr)
        sys.exit(1)

    showOrderNum = args.showOrderNum == "true"
    timestart = datetime.datetime.now().timestamp()
    filePathName = getFilePathName(args)
    token = args.token
    if not token:
        token = os.getenv("GITHUB_TOKEN")

    printProgress(0)
    gh = Github(token, retry=retryConfig()) if args.token else Github(retry=retryConfig())
    user = gh.get_user(args.username)
    printProgress(1)
    starred = []
    for repo in starred_repos(user):
        if repo.private and args.excludeprivate == "true":
            continue
        starred.append(repo)

    StarsExporter.exportToFile(args, starred, filePathName, showOrderNum)

    printProgress(100)
    print("\nExport completed, time cost:", str(int(datetime.datetime.now().timestamp() - timestart)), "secs.")


if __name__ == "__main__":
    main()
